#I have often wished for an easy and general function in S for
#univariate description.  stem() is close, but it does not give
#mean and standard deviation nor take advantage of graphics
#abilities.  It also has a fairly dense way of
#handling discrete data.  
#
#If dplot=T, descr() gives a "pretty" histogram, with axes for percentages
#as well as counts, and the percentages printed above each bar. 
#
#If dplot=F, you get a stem-and leaf.
#
#In both instances:
#    mean s.d. extremes and quartiles are printed;
#    missing values are allowed but noted;
#    discrete variables are handled intelligently.
#
#Please, use enjoy and give me feedback.  No restrictions.
#
#Rick Chappell

descr <- function(x,discrete=F,dplot=F,reset=T) {

# x is a vector about which you want summary statistics
#        (missing values are permitted).
#
# The mean, standard deviation, and 0th, 25th, 50th, 75th, 100th percentiles
#        are calculated and number of missing values given.
#
# If discrete=T, then descr() assumes that x is integer-valued, and draws
#        and labels the stem-and-leaf plot and histogram accordingly
#
# If dplot=F, then these statistics are stated along with a stem-and-leaf plot.
# 
# If dplot=T and a graphics device is open, then you get these stats along
#        with an "improved" histogram.  The improvements consist of an extra
#        axis on the right labelled with percentages and percentages printed
#        on top of each bar.
#
# Format when dplot=T:
#        When reset=T, descr() divides the page vertically in two.  The "top"
#        plot gives the summary statistics; the bottom has the histogram.
#        If you are using the postscript device driver, it automatically 
#        restores the par(mfrow) field to its prior value.  Otherwise, it asks
#        before doing so (and clearing the screen).
#
#        When reset=F, the pagination is governed by the current graphics
#        parameters; descr() does not need to ask you about resetting,
#        making reset=F useful for batch jobs.  If no parameters have
#        been set, then you will get the summary statistics on one page
#        and the histogram on the next.  

if(!dplot) {
cat("Mean   =  ",format(signif(mean(x[!is.na(x)]),4)),"\n" )
cat("Standard deviation   =  ",format(signif(sqrt(var(x[!is.na(x)])),4)),"\n" )
if (discrete) stem(x,scale=0) else stem(x)
          } else

         {
           if(reset) assign("oldpar",par(mfrow=c(2,1)))

mm <- sum(is.na(x))
x <- x[!is.na(x)]

if (mm==1) assign("miss", "There is 1 missing value") else
                assign("miss", paste("There are",format(mm),"missing values"))
plot(1:10,1:10,xlab="",ylab="",xaxt="n",yaxt="n",bty="n",type="n")
title(main=paste("Descriptive Statistics For \n",deparse(substitute(x))),
      sub=miss)


text(6,9.5,labels=paste("  = ",format(signif(mean(x),4))),adj=0)
text(6,8,labels=paste("  = ",format(signif(sqrt(var(x)),4))),adj=0)
qq <- format(signif(quantile(x),4))
text(6,6,labels=paste("  = ",qq[5]),adj=0)
text(6,4.5,labels=paste("  = ",qq[4]),adj=0)
text(6,3,labels=paste("  = ",qq[3]),adj=0)
text(6,1.5,labels=paste("  = ",qq[2]),adj=0)
text(6,0,labels=paste("  = ",qq[1]),adj=0)

text(6,9.5,labels="Mean",adj=1)
text(6,8,labels="Standard deviation",adj=1)
text(6,6,labels="Maximum",adj=1)
text(6,4.5,labels="Upper quartile",adj=1)
text(6,3,labels="Median",adj=1)
text(6,1.5,labels="Lower quartile",adj=1)
text(6,0,labels="Minimum",adj=1)

if (discrete) {
bb <- (min(x) - 1):(max(x))
hist(x,xlab="",breaks=bb,xaxt="n",names=as.vector(bb[-1],mode=CHAR))
hh <- hist(x,plot=F,breaks=bb)
              } else
{hist(x,xlab="")
hh <- hist(x,plot=F)}
ll <- len(hh$breaks)
pp <- 100*hh$counts/sum(hh$counts)
text(hh$breaks[-ll] + (hh$breaks[2] - hh$breaks[1])/2,
     hh$counts + max(hh$counts)/10,
     labels=paste(" ",format(signif(pp,digits=2)),"%"),adj=.5,cex=.5)

par(new=T)
plot(hh$breaks[-ll],pp,type="n",xlab="",ylab="",bty="n",xaxt="n",yaxt="n")
pp <- pretty(pp)
axis(side=4,at=pp,labels=paste(" ",format(pp),"%"))

if(.Device != "postscript" & reset) {
cat("Are you ready to reset the parameters? (no)   ")
nn <- readline()
if(nn=="yes" | nn=="y") par(oldpar) else
    cat("Graphical parameters not restored; old parameters were mfrow =",
         oldpar,"\n")
                                    } 

if(.Device == "postscript" & reset) {par(oldpar)} else {}
         }                          }

